/*
 * Declare list.cc and macros.
 * See source file for details.
 */



#ifndef __BASE_LIST_H__
#define __BASE_LIST_H__



// list nodes
class class_node {

	public:

	class_node * next;	// point next node

	class_node *previous;	// point previous node

	DATAPT data;		// user data

};



// nodes can be inserted after or before one specific position
enum relative_location {
	BEFORE,
	AFTER
};



// manipulate the list
class class_list {

	public:

	class_list ();
	~class_list ();

	protected:

	class_node * m_first;	// first node

	class_node *m_last;	// last node

	class_node *m_selected;	// node selected for access

	DWORD m_total_nodes;	// number of nodes in list

	class_node **m_direct_access;	// array of pointers for direct access

	// point comparison function between two given nodes data
	      DATAPT (*compare_function) (DATAPT, DATAPT);

	// internal funcions
	inline class_node *get_node_at_position (DWORD);
	inline void insert_node (class_node *, relative_location,
				 class_node *);
	inline void remove_node (class_node *);
	inline void append_list (class_list &, relative_location, class_node *);
	inline class_node *make_btree_search (relative_location, DATAPT, DWORD &);
	inline class_node *make_linear_search (relative_location, class_node *, DATAPT);

	// external user functions
	      public:

	void  init ();
	void  terminate ();

	// functions to remove nodes
	void  remove_all (BOOL = FALSE);
	BOOL  remove_first (DATAPT &);
	BOOL  remove_last (DATAPT &);
	BOOL  remove_selected (DATAPT &);
	BOOL  remove_at_position (DWORD, DATAPT &);

	// functions to insert nodes
	BOOL  insert_first (relative_location, DATAPT);
	BOOL  insert_last (relative_location, DATAPT);
	BOOL  insert_selected (relative_location, DATAPT);
	BOOL  insert_at_position (relative_location, DWORD, DATAPT);

	// functions to access nodes
	BOOL  get_first (DATAPT &);
	BOOL  get_last (DATAPT &);
	BOOL  get_selected (DATAPT &);
	BOOL  get_at_position (DWORD, DATAPT &);

	// functions to set the selected node
	void  select_first ();
	void  select_last ();
	void  select_at_position (DWORD);
	void  select_next (DWORD = 1);
	void  select_previous (DWORD = 1);
	void  select_offset (int);
	void  select_none ();

	// functions to swap node data
	BOOL  swap_first (DATAPT &);
	BOOL  swap_last (DATAPT &);
	BOOL  swap_selected (DATAPT &);
	BOOL  swap_at_position (DWORD, DATAPT &);

	// functions to append lists to this list
	void  append_first (relative_location, class_list &);
	void  append_last (relative_location, class_list &);
	BOOL  append_selected (relative_location, class_list &);
	void  append_at_position (DWORD, relative_location, class_list &);

	// functions to get list status
	DWORD get_total_nodes ();
	BOOL  has_selection ();
	BOOL  get_select_position (DWORD &);

	// functions to manipulate de hole list
	void  do_circular ();
	void  do_linear ();
	BOOL  enable_direct_access ();
	void  disable_direct_access ();
	void  revert ();
	void  sort ();
	DWORD find_sorted (relative_location, DATAPT);
	void  find_unsorted (relative_location, DATAPT);
	BOOL  split (DWORD, DWORD, class_list &);

	// functions to set the aplly functions
	void  set_compare_function (DATAPT (*) (DATAPT, DATAPT));
};



// macros

#define alloc_node \
(class_node*) malloc (sizeof (class_node));

#define free_node(NODE) \
free (NODE);

#define set_data(NODE, DATA) \
{  if ( (NODE) != NULL ) (NODE)->data = (DATAPT) (DATA);  }

#define get_data(NODE) \
((NODE)->data)



// end declarations
#endif
